  # __________________________________________________________ #
  #                                |                           #
  #  이토록 쉬운                   |     "데이터는 답이다"     #
  #  데이터 요약과 시각화 with R   |       _                   #
  # ______________________________ |    __| |   __ _   _ __    #
  #                                |   / _` |  / _` | | '_ \   #
  #  임경덕                        |  | (_| | | (_| | | |_) |  # 
  #   - dataartproject@gmail.com   |   \__,_|  \__,_| | .__/   #
  #   - http://dataartproject.com  |                  |_|      #
  # _______________________________|__________________________ #



# 5장 | 다양한 데이터 요약과 시각화

# 다양한 분석 사례




# 5.0 예제 데이터 불러오기 -------------------------------------------------------

  ## 필요에 따라 아래의 [코드 4-1] 재실행


# [코드 4-1] 
checkout = read.csv("data/checkout.csv", fileEncoding = "UTF-8")
customer = read.csv("data/customer.csv", fileEncoding = "UTF-8")
merchant = read.csv("data/merchant.csv", fileEncoding = "UTF-8")


# [코드] 패키지 불러오기
library(dplyr)
library(ggplot2)


# macOS 사용자만 한글 폰트 설정
theme_set(theme_gray() + theme(text=element_text(family='AppleGothic')))




# 5.1 변수를 몇 개만 보거나 숨기고 싶어요 ------------------------------------------------


# [코드 5-1]
checkout %>% 
  tibble()


checkout %>%
  select(merc_id, datetime, amount) %>% 
  tibble()



# [코드 5-2]
checkout2 = checkout


checkout2 = checkout2 %>%
  select(merc_id, datetime, amount)


checkout2 %>% 
  head()



# [코드 5-3]
checkout_sub = checkout %>%
  select(merc_id, datetime, amount)



# [코드 5-4]
checkout %>%
  select(-datetime) %>% 
  tibble()



# [코드 5-5]
checkout %>%
  select(merc_id:amount) %>% 
  tibble()



# [코드 5-6]
checkout %>%
  select(ends_with("id"), contains("am")) %>% 
  tibble()



# [코드 5-7]
checkout %>%
  select( matches("[a-zA-Z]{7,}")) %>% 
  tibble()




# [코드 5-8]
checkout %>%
  select(1, 3:5) %>% 
  tibble()



# [코드 5-9]
system.time(
  checkout %>% 
    select(cust_id, category, amount) %>% 
    filter(category %in% c("백화점", "대형마트", "슈퍼마켓")) %>% 
    group_by(cust_id, category) %>% 
    summarise(n = n(), total = sum(amount)) %>% 
    arrange(cust_id, desc(total)) 
)





# 5.2 연령대 변수를 만들어서 분석하고 싶어요 -----------------------------------------------


# [코드 5-10]
customer %>% 
  mutate(new_variable = 1) %>% 
  tibble()



# [코드 5-11]
1:10
cut(1:10, breaks = c(0, 3, 7, 10))



# [코드 5-12]
cut(1:10, breaks = c(0, 3, 7, 10), labels = c("A","B","C"), right = F)



# [코드 5-13]
cut(1:10, breaks = c(0, 3, 7, 20), labels = c("A","B","C"), right = F, include.lowest = T)



# [코드 5-14]
customer %>%
  mutate(연령대 = cut(age,
                   breaks=seq(15, 100, 5),
                   labels=paste0(seq(15, 95, 5), "_"),
                   right=F)) %>%
  tibble()



# [코드 5-15]
customer %>% 
  tibble()


customer = customer %>%
  mutate(연령대 = cut(age,
                   breaks = seq(15, 100, 5),
                   labels = paste0(seq(15, 95, 5), "_"),
                   right = F))


customer %>% 
  tibble()



# [코드 5-16]
연령대별고객수 = customer %>%
  group_by(연령대) %>% 
  summarise(n = n())

연령대별고객수



# [코드 5-17]
library(ggplot2)
연령대별고객수 %>% 
  ggplot(aes(연령대, n)) + 
  geom_col()



# [코드 5-18]
고객실적구간 = checkout %>% 
  group_by(cust_id) %>% 
  summarise(total = sum(amount)) %>% 
  mutate( 
    실적구간 = cut(total,
               breaks = c(-1, 0, 100000, 500000, 1000000, 9999999),
               labels = c("1_무실적", "2_10만원이하", "3_50만원이하", "4_100만원이하", "5_100만원이상"),
               right = T))

고객실적구간



# [코드 5-19]
고객실적구간 %>% 
  group_by(실적구간) %>% 
  summarise(n = n()) %>% 
  ggplot(aes(실적구간, n)) + 
  geom_col()






# 5.3 비어 있는 결측치를 채우고 싶어요 --------------------------------------------------


# [코드 5-20]
na_data = read.csv("data/data_na.csv")
na_data



# [코드 5-21]
na_data$var1



# [코드 5-22]
na_data %>% 
  summarise(n       = n(),
            total   = sum(var2),
            average = mean(var2))



# [코드 5-23]
1+NA



# [코드 5-24]
na_data %>% 
  summarise(n       = n(),
            total   = sum(var2, na.rm = T),
            average = mean(var2, na.rm = T))



# [코드 5-25]
na_data %>% 
  group_by(var1) %>% 
  summarise(total = sum(var2, na.rm = T))



# [코드 5-26]
고객실적요약 = checkout %>% 
  group_by(cust_id) %>% 
  summarise(total = sum(amount))

고객실적요약



# [코드 5-27]
고객정보 = customer %>% 
  select(cust_id, gender, age, 연령대)

고객실적요약전체 = merge(고객정보, 고객실적요약, by = "cust_id", all.x = T)
고객실적요약전체 %>% 
  tibble()



# [코드 5-28]
고객실적요약전체 %>% 
  summarise(average = mean(total, na.rm = T))



# [코드 5-29]
is.na(na_data$var2)



# [코드 5-30]
subdata_na = 고객실적요약전체 %>% 
  filter(is.na(total)) 

subdata_notna = 고객실적요약전체 %>% 
  filter(!is.na(total)) 



# [코드 5-31]
if_else(3>2, "참입니다", "거짓입니다")
if_else(3<2, "참입니다", "거짓입니다")



# [코드 5-32]
고객실적요약전체 %>% 
  mutate(total2 = if_else(is.na(total), 0, total)) %>% 
  tibble()



# [코드 5-33]
고객실적요약전체 %>% 
  mutate(total2 = if_else(is.na(total), 0, total)) %>% 
  summarise(average = mean(total2))



# [코드 5-34]
고객실적요약전체 %>% 
  ggplot(aes(total)) + 
  geom_histogram(binwidth = 50000)



# [코드 5-35]
고객실적요약전체 %>% 
  mutate(total2 = if_else( is.na(total), 0, total )) %>% 
  ggplot(aes(total2)) + 
  geom_histogram(binwidth = 50000)



# [코드 5-36]
고객실적구간 = 고객실적요약전체 %>% 
  mutate(total2 = if_else( is.na(total), 0, total )) %>% 
  mutate( 
    실적구간 = cut(total2,
               breaks=c(-1, 0, 100000, 500000, 1000000, 9999999),
               labels=c("1_무실적", "2_10만원이하", "3_50만원이하", "4_100만원이하", "5_100만원이상"),
               right=T)) 
고객실적구간 %>% tibble()


고객실적구간 %>% 
  group_by(실적구간) %>% 
  summarise( n = n()) %>% 
  ggplot(aes(실적구간, n)) + 
  geom_col()






# 5.4 다양한 조건으로 그룹 변수를 만들어볼까요? ---------------------------------------------


# [코드 5-37]
customer %>% 
  mutate(target = if_else(between(age, 20, 39.9) & gender == 2, "Y", "N")) %>% 
  count(target)



# [코드 5-38]
customer %>% 
  mutate(target = if_else(between(age, 20, 39.9) & gender==2, "1_주사용자", 
                          if_else(between(age, 30, 39.9) & gender==1, "2_관심사용자", "3_나머지"))) %>% 
  count(target)



# [코드 5-39]
customer %>%
  mutate(고객군 = case_when(
    between(age, 20, 39.9) & gender == 2 ~ "GRP1_20F30F", 
    between(age, 30, 39.9) & gender == 1 ~ "GRP2_30M",
    between(age, 20, 49.9) ~ "GRP3_20M40",
    TRUE ~ "GRP4_ETC")) %>%
  count(고객군)



# [코드 5-40]
customer = customer %>%
  mutate( 고객군 = case_when(
    between(age, 20, 39.9) & gender == 2 ~ "GRP1_20F30F", 
    between(age, 30, 39.9) & gender == 1 ~ "GRP2_30M",
    between(age, 20, 49.9) ~ "GRP3_20M40",
    TRUE ~ "GRP4_ETC")) 



# [코드 5-41]
merged1 = merge(checkout, customer, by="cust_id") 

merged1 %>% 
  filter(category == "배달업종") %>% 
  group_by(고객군) %>% 
  summarise(n_cust        = n_distinct(cust_id),
            mean_checkout = n() / n_distinct(cust_id),
            average       = mean(amount))



# [코드 5-42]
merged1 = merge(checkout %>% filter(category == "배달업종"), customer, by = "cust_id") 


merged1 %>% 
  filter(category == "배달업종") %>% 
  group_by(고객군) %>% 
  summarise(n_cust        = n_distinct(cust_id),
            mean_checkout = n() / n_distinct(cust_id),
            average       = mean(amount))





# 5.5 고객별로 결제 금액이 가장 큰 업종을 찾고 싶어요 -----------------------------------------


# [코드 5-43]
checkout %>% 
  slice_max(amount, n = 5) 



# [코드 5-44]
checkout %>% 
  slice_max(amount, prop = 0.0001) 



# [코드 5-45]
checkout %>%
  group_by(cust_id) %>%
  summarise(total = sum(amount)) %>%
  slice_max(total, n = 3)



# [코드 5-46]
checkout %>%
  group_by(cust_id) %>%
  summarise(total = sum(amount)) %>%
  slice_min(total, n = 3)



# [코드 5- 추가 함수 소개]
checkout %>% 
  slice(3:5) 

checkout %>% 
  slice_head(n = 3)

checkout %>% 
  slice_head(prop = 0.0001)

checkout %>% 
  slice_tail(n = 3)

checkout %>% 
  slice_sample(n = 3)




# [코드 5-47]
checkout %>%
  group_by(cust_id) %>%
  slice_max(datetime, n = 1)



# [코드 5-48]
agg = checkout %>%
  group_by(category, merc_id) %>%
  summarise(sales = sum(amount))
agg



# [코드 5-49]
agg %>% 
  slice_max(sales, n = 1) %>% 
  print(n = 30)



# [코드 5-50]
checkout %>%
  group_by(merc_id, category) %>%
  summarise(sales = sum(amount)) %>% 
  group_by(category) %>% 
  slice_max(sales, n = 1)







# 5.6 업종별로 돈을 제일 많이 쓴 고객을 살펴봅시다 -------------------------------------------


# [코드 5-51]
agg_top_cust = checkout %>% 
  group_by(category, cust_id) %>% 
  summarise(total = sum(amount)) %>% 
  slice_max(total, n = 1) 

agg_top_cust %>% 
  print(n = 30)





  
# 5.7 결제 금액이 가장 큰 고객의 정보를 확인하고 싶어요 ----------------------------------------


# [코드 5-52]
merge(agg_top_cust, customer, by = "cust_id") 



# [코드 5-53]
customer %>% 
  filter(cust_id %in% c("C18681", "C17645", "C13664"))



# [코드 5-54]
agg_top_cust$cust_id



# [코드 5-55]
customer %>% 
  filter(cust_id %in% agg_top_cust$cust_id) 



# [코드 5-56]
merc_region = merchant %>% 
  filter(grepl("분석로1길", address)) 

merc_region %>% 
  slice_sample(n = 5)



# [코드 5-57]
checkout_region = checkout %>% 
  filter(merc_id %in% merc_region$merc_id)

checkout_region %>% 
  slice_sample(n = 5) 



# [코드 5-58]
checkout_region %>% 
  summarise(n      = n(),
            n_cust = n_distinct(cust_id))



# [코드 5-59]
cust_region1 = checkout_region$cust_id
length(cust_region1)


cust_region2 = unique(checkout_region$cust_id)
length(cust_region2)


customer_region = customer %>% 
  filter(cust_id %in% cust_region2)

customer_region %>% 
  slice_sample(n = 5)






# 5.8 주소를 쪼개서 지역별로 분석해봅시다 -------------------------------------------------


# [코드 5-60]
merchant %>% 
  tibble()



# [코드 5-61]
merchant %>% 
  mutate(시군구 = substr(address, 1, 4)) %>% 
  select(1:4, 7) %>% 
  tibble()



# [코드 5-62]
merchant %>% 
  mutate(시군구 = substr(address, 1, 4)) %>% 
  select(시군구) %>% 
  unique( )



# [코드 5-63]
merchant %>% 
  mutate(시군구 = substr(address, 1, 4)) %>% 
  distinct(시군구) 



# [코드 5-64]
merchant %>% 
  mutate( 도로명 = substr(address, 6, 10)) %>% 
  distinct(도로명) 



# [코드 5-65]
strsplit("서울시 강동구 우리집로 1234", split=" ")



# [코드 5-66]
install.packages("tidyr")
library(tidyr)


merchant %>%
  separate(address, into=c("시군구", "도로명", "건물번호"), sep=" ") %>% 
  tibble()



# [코드 5-67]
merchant %>%
  separate(address, into=c("시군구", "도로명"), sep=" ") %>% 
  tibble()



# [코드 5-68]
merchant %>%
  filter(name != "")  %>% 
  tibble()



# [코드 5-69]
merchant %>%
  filter(name != "")  %>% 
  separate(name, into=c("brand")) %>% 
  tibble()



# [코드 5-70]
checkout_cvs = checkout %>% 
  filter(category == "편의점")


merchant_cvs = merchant %>%
  filter(category == "편의점")  %>% 
  separate(name, into=c("brand")) %>% 
  select(merc_id, brand)



# [코드 5-71]
data_cvs = merge(checkout_cvs, merchant_cvs, by="merc_id")
data_cvs %>% 
  tibble()



# [코드 5-72]
data_cvs %>% 
  group_by(cust_id, brand) %>% 
  summarise(n = n())



# [코드 5-73]
data_cvs %>% 
  group_by(cust_id, brand) %>% 
  summarise(n_checkout = n()) %>% 
  slice_max(n_checkout, n = 1)







# 5.9 1, 2 대신 남, 여로 바꿔야 이해하기 편합니다 -----------------------------------------


# [코드 5-74]
customer %>% 
  group_by(gender) %>% 
  summarise(n = n())



# [코드 5-75]
customer %>% 
  group_by(gender) %>% 
  summarise(n = n()) %>% 
  ggplot(aes(gender, n)) + geom_col()



# [코드 5-76]
customer %>% 
  group_by(gender) %>% 
  summarise(n = n()) %>% 
  ggplot(aes(gender, n, fill=gender)) + geom_col()



# [코드 5-77]
customer %>% 
  mutate( gender = case_when(
    gender == 1 ~ "1_M",
    gender == 2 ~ "2_F")) %>% 
  group_by(gender) %>% 
  summarise(n = n())



# [코드 5-78]
customer %>% 
  mutate( gender = case_when(
    gender==1 ~ "1_M",
    gender==2 ~ "2_F")) %>% 
  group_by(gender) %>% 
  summarise(n = n()) %>% 
  ggplot(aes(gender, n, fill = gender)) + geom_col()





# 5.10 그룹에 따라 결제 금액 분포가 어떻게 다를까요? -----------------------------------------


# [코드 5-79]
checkout_cust = checkout %>% 
  group_by(cust_id) %>% 
  summarise(total = sum(amount))


cust_checkout = merge(checkout_cust, customer %>% select(1:3), by="cust_id")
cust_checkout %>% 
  tibble()



# [코드 5-80]
cust_checkout = cust_checkout %>%
  mutate(연령대_10 = cut(
           age,
           breaks = seq(20, 90, 10),
           labels = paste0(seq(20, 80, 10), "대"),
           right = F),
         gender = case_when(
           gender == 1 ~ "1_남",
           gender == 2 ~ "2_여"))

cust_checkout %>% 
  tibble()



# [코드 5-81]
cust_checkout %>% 
  ggplot(aes(x = total)) + geom_histogram()

cust_checkout %>% 
  ggplot(aes(y = total)) + geom_boxplot()



# [코드 5-82]
cust_checkout %>% 
  ggplot(aes(x = total, fill = gender)) + geom_histogram()



# [코드 5-83]
cust_checkout %>% 
  ggplot(aes(x = total, fill = gender)) + 
  geom_histogram(position = "identity", alpha = 0.5 ) +
  scale_fill_brewer(palette = "Set1", direction = -1)



# [코드 5-84]
cust_checkout %>% 
  ggplot(aes(x = total, fill = 연령대_10)) + 
  geom_histogram(position = "identity", alpha = 0.2) 



# [코드 5-85]
cust_checkout %>% 
  ggplot(aes(y = total, x = gender)) + 
  geom_boxplot() 



# [코드 5-86]
cust_checkout %>% 
  ggplot(aes(y = total, x = 연령대_10)) + 
  geom_boxplot() 



# [코드 5-87]
cust_checkout %>% 
  ggplot(aes(y = total, x = 연령대_10, fill = gender)) + 
  geom_boxplot() 



# [코드 5-88]
cust_checkout %>% 
  group_by(연령대_10, gender) %>% 
  summarise(total = mean(total))







# 5.11 두 개 그룹을 활용한 요약을 그래프로 표현해봅시다 ----------------------------------------


# [코드 5-89]
cust_checkout %>% 
  group_by(연령대_10) %>% 
  summarise(med_total = median(total)) %>% 
  ggplot(aes(연령대_10, med_total)) + 
  geom_col()


# [코드 5-90]
cust_checkout %>% 
  group_by(연령대_10, gender) %>% 
  summarise(med_total = median(total)) %>% 
  tibble()



# [코드 5-91]
cust_checkout %>% 
  group_by(연령대_10, gender) %>% 
  summarise(med_total = median(total)) %>% 
  ggplot(aes(연령대_10, med_total, fill = gender)) + 
  geom_col(position = "dodge") + 
  scale_fill_brewer(palette = "Set1", direction = -1)



# [코드 5-92]
cust_checkout %>% 
  group_by(연령대_10, gender) %>% 
  summarise(med_total = median(total)) %>% 
  ggplot(aes(gender, med_total, fill = 연령대_10)) + 
  geom_col(position = "dodge") 



# [코드 5-93]
cust_checkout %>%
  group_by(연령대_10, gender) %>%
  summarise(med_total = median(total)) %>%
  ggplot(aes(gender, med_total)) +
  geom_col()



# [코드 5-94]
cust_checkout %>% 
  group_by(연령대_10, gender) %>% 
  summarise(med_total = median(total)) %>% 
  ggplot(aes(gender, med_total, fill = 연령대_10)) + 
  geom_col() 





# 5.12 연령대별 선호 업종을 찾고 히트맵으로 표현해봅시다 ----------------------------------------


# [코드 5-95]
checkout2 = merge(checkout, customer %>% select(1:3), by = "cust_id")
checkout2 %>% 
  tibble()



# [코드 5-96]
checkout2 = checkout2 %>%
  mutate(연령대_10 = cut(age,
                      breaks = seq(20, 90, 10),
                      labels = paste0(seq(20, 80, 10), "대"),
                      right = F, include.lowest = T))


checkout2 %>% 
  group_by(연령대_10, category) %>% 
  summarise(total = sum(amount))



# [코드 5-97]
checkout2 %>% 
  group_by(연령대_10, category) %>% 
  summarise(total = sum(amount)) %>% 
  ggplot(aes(연령대_10, total, fill = category))+
  geom_col(position = "dodge")



# [코드 5-98]
checkout2 %>% 
  group_by(연령대_10, category) %>% 
  summarise(total = sum(amount)) %>% 
  group_by(연령대_10) %>% 
  summarise(grp_total = sum(total))



# [코드 5-99]
checkout2 %>% 
  group_by(연령대_10, category) %>% 
  summarise(total = sum(amount)) %>% 
  group_by(연령대_10) %>% 
  mutate(grp_total = sum(total)) 



# [코드 5-100]
checkout2 %>% 
  group_by(연령대_10, category) %>% 
  summarise(total = sum(amount)) %>% 
  group_by(연령대_10) %>% 
  mutate(grp_total = sum(total)) %>% 
  mutate(prop_category = total / grp_total)



# [코드 5-101]
checkout2 %>% 
  group_by(연령대_10, category) %>% 
  summarise(total = sum(amount)) %>% 
  group_by(연령대_10) %>% 
  mutate(grp_total = sum(total),
         prop_category = total / grp_total) 



# [코드 5-102]
category_prop_by_grp = checkout2 %>% 
  group_by(연령대_10, category) %>% 
  summarise(total = sum(amount)) %>% 
  group_by(연령대_10) %>% 
  mutate(prop_category = total / sum(total)) 



# [코드 5-103]
checkout2 %>% 
  group_by(연령대_10, category) %>% 
  summarise(total = sum(amount)) %>% 
  group_by(연령대_10) %>% 
  mutate(rel_category = total / max(total)) 



# [코드 5-104]
category_prop_by_grp %>% 
  ggplot(aes(연령대_10, prop_category, fill = category)) + 
  geom_col()



# [코드 5-105]
checkout2 %>% 
  group_by(연령대_10, category) %>% 
  summarise(total = sum(amount)) %>% 
  ggplot(aes(연령대_10, total, fill = category)) + 
  geom_col(position = "fill")



# [코드 5-106]
category_prop_by_grp %>% 
  ggplot(aes(연령대_10, category, fill = total)) + 
  geom_tile() +
  scale_fill_distiller(palette = "Blues", direction = 1)



# [코드 5-107]
category_prop_by_grp %>% 
  ggplot(aes(연령대_10, category, fill = prop_category)) + 
  geom_tile() +
  scale_fill_distiller(palette = "Blues", direction = 1)



# [코드 5-108]
checkout2 %>% 
  filter(gender == "1") %>% 
  group_by(연령대_10, category) %>% 
  summarise(MEAN = mean(amount),
            n = n()) %>% 
  ggplot(aes(연령대_10, category, fill = MEAN, label = n)) + 
  geom_tile() +
  scale_fill_distiller(palette = "Blues", direction = 1) +
  geom_text()







# 5.13 일별 매출 추이를 그리고 요일별로 분석합시다 -------------------------------------------


# [코드 5-109]
checkout %>% 
  tibble()



# [코드 5-110]
dt1 = "2030-01-01 11:22:33"
strptime(dt1, format = "%Y-%m-%d %H:%M:%S")



# [코드 5-111]
strptime(dt1, format = "%F %T")



# [코드 5-112]
dt2 = "12/31/2030 01:22"
strptime(dt2, format = "%m/%d/%Y %H:%M")



# [코드 5-113]
dt3 = "300101112233"
strptime(dt3, format = "%y%m%d%H%M%S")



# [코드 5-114]
checkout %>% 
  mutate( datetime = strptime(datetime, "%Y-%m-%d %H:%M:%S")) %>% 
  tibble() 



# [코드 5-115]
as.Date("12/31/2030", format="%m/%d/%Y")



# [코드 5-116]
checkout = checkout %>% 
  mutate(datetime = strptime(datetime, "%Y-%m-%d %H:%M:%S"),
         date     = as.Date(datetime))

checkout %>% 
  tibble()



# [코드 5-117]
checkout %>% 
  group_by(date) %>% 
  summarise(total = sum(amount))



# [코드 5-118]
checkout %>% 
  group_by(date) %>% 
  summarise(total = sum(amount)) %>% 
  ggplot(aes(date, total)) + 
  geom_line()



# [코드 5-119]
checkout %>% 
  group_by(date) %>% 
  summarise(total = sum(amount)) %>% 
  ggplot(aes(date, total)) + 
  geom_line() + 
  scale_x_date(date_labels = "%m-%d", date_breaks = "1 day")



# [코드 5-120]
checkout %>% 
  group_by(date) %>% 
  summarise(total = sum(amount)) %>% 
  mutate(cum_total = cumsum(total)) %>% 
  ggplot(aes(date, cum_total)) + 
  geom_line()+ 
  scale_x_date(date_labels = "%m-%d", date_breaks = "1 day")



# [코드 5-121]
checkout %>% 
  group_by(date, category) %>% 
  summarise(total = sum(amount)) %>% 
  group_by(category) %>% 
  mutate(cum_total = cumsum(total)) %>% 
  ggplot(aes(date, cum_total, color = category)) + 
  geom_line()+ 
  scale_x_date(date_labels = "%m-%d", date_breaks = "1 week")



# [코드 5-122]
checkout %>% 
  mutate( month = format(datetime, format="%m"),
          hour  = format(datetime, format="%H"),
          dow1  = format(date, format="%a"), 
          dow2  = format(date, format="%u"), 
          dow   = format(date, format="%u_%a")) %>% 
  select(datetime, month, hour, dow1, dow2, dow) %>% 
  sample_n(5) %>% 
  tibble()



# [코드 5-123]
agg_dow_hour = checkout %>% 
  mutate(hour = format(datetime, format = "%H"),
         dow  = format(date, format = "%u_%a")) %>% 
  group_by(dow, hour) %>% 
  summarise(n_cust = n_distinct(cust_id),
            total  = sum(amount))

agg_dow_hour



# [코드 5-124]
agg_dow_hour %>% 
  ggplot(aes(dow, hour, fill = total, label = n_cust)) +
  geom_tile() +
  scale_fill_distiller(palette = "YlGnBu", direction = 1) +
  geom_text() 



# [코드 5-125]
agg_dow_category = checkout %>% 
  filter(!(category %in% c("아파트관리비", "도시가스비", "대중교통"))) %>% 
  mutate(hour = format(datetime, format = "%H"),
         dow  = format(date, format = "%u_%a")) %>% 
  group_by(category, dow) %>% 
  summarise(n_cust = n_distinct(cust_id),
            total  = sum(amount)) %>% 
  mutate(prop = total / sum(total))


agg_dow_category %>% 
  ggplot(aes(dow, category, fill = prop)) +
  geom_tile() +
  scale_fill_distiller(palette = "YlGnBu", direction = 1) 







# 5.14 가맹점의 매출 건수와 매출 금액의 관계를 살펴봅시다 ---------------------------------------


# [코드 5-126]
agg_merc = checkout %>% 
  group_by(merc_id) %>% 
  summarise(n = n(),
            total = sum(amount))
agg_merc


agg_merc %>% 
  ggplot(aes(n, total)) + 
  geom_point(alpha = 0.3)



# [코드 5-127]
agg_merc_1000 = agg_merc %>% 
  filter(n >= 1000)
agg_merc_1000



# [코드 5-128]
agg_merc %>% 
  ggplot(aes(n, total)) + 
  geom_point(alpha = 0.3) +
  geom_text(data = agg_merc_1000, aes(label = merc_id), vjust=1)



# [코드 5-129]
agg_merc %>% 
  mutate(col_var = if_else(n >= 1000, "target", "normal")) %>% 
  ggplot(aes(n, total)) + 
  geom_point(alpha = 0.5, size = 2, aes(color = col_var)) +
  geom_text(data = agg_merc_1000, aes(label = merc_id), hjust = 1.2)



# [코드 5-130]
agg_merc %>% 
  filter(n < 500, total < 2000000) %>% 
  ggplot(aes(n, total)) + 
  geom_point(alpha = 0.3)



# [코드 5-131]
checkout %>% 
  group_by(merc_id, category) %>% 
  summarise(n = n(),
            total = sum(amount)) %>% 
  filter(n < 500, total < 2000000) %>% 
  ggplot(aes(n, total, color = category)) + 
  geom_point(alpha = 0.5)



# 상관계수 관련 추가 예제
  ## 예제 데이터 : 1078 가족의 아빠키, 아들키 데이터
df_heights = read.csv('data/practice/heights.csv')
df_heights %>% head()

  ## 산점도 그리기
  df_heights %>% 
    ggplot(aes(father, son)) + 
    geom_point()
  
  ## cor( ) 함수를 활용한 상관계수 계산
  df_heights %>% 
    summarise(COR = cor(father, son))
    ### 아빠키-아들키의 상관계수는 +0.5

  
# 상관계수 추가 예제
  ## 결제 건수와 결제 금액의 상관계수 계산

  ## 전체 가맹점의 결제 건수와 결제 금액으로 계산한 상관계수
  checkout %>% 
    group_by(merc_id, category) %>% 
    summarise(n = n(),
              total = sum(amount)) %>% 
    ungroup() %>%   ## 그룹 변수 지정 해제
    summarise(COR = cor(n, total))
  
  
  
  ## category별로 나눠서 계산한 상관계수
  checkout %>% 
    group_by(merc_id, category) %>% 
    summarise(n = n(),
              total = sum(amount)) %>% 
    group_by(category) %>% 
    summarise(COR = cor(n, total))
  
  
  ## 해석 : 전체 상관계수는 0.746으로 양의 관계를 확인할 수 있으나
  ##        category별로 나눠 상관계수를 계산해보면 대부분 0.95 이상으로 
  ##        동일 업종 내 결제 건수와 결제 금액에는 매우 강한 양의 상관이 있다고 볼 수 있음
  



  

# 5.15 관측치를 나눠서 그래프를 그리면 뭔가 보입니다 ------------------------------------------


# [코드 5-132]
cust_checkout = merge(checkout, customer %>% select(1:3), by="cust_id")
  
cust_checkout = cust_checkout %>%
  mutate(연령대_10 = cut(
           age,
           breaks = seq(20, 90, 10),
           labels = paste0(seq(20, 80, 10), "대"),
           right = F, include.lowest = T),
         성별 = case_when(
           gender == 1 ~ "1_남",
           gender == 2 ~ "2_여"),
         datetime = strptime(datetime, "%Y-%m-%d %H:%M:%S"),
         결제시간 = format(datetime, format = "%H"),
         요일     = format(datetime, format = "%u_%a"))
  ## 요일의 경우 언어 설정에 따라 책과 다르게 '1_Mon', '6_Sat'과 같이 생성될 수 있음



# [코드 5-133]
cust_checkout %>% 
  filter(category == "커피/음료전문점") %>% 
  ggplot(aes(amount, fill = 성별)) +
  geom_histogram(position = "identity", alpha = 0.5)



# [코드 5-134]
cust_checkout %>% 
  filter(category == "커피/음료전문점") %>% 
  filter(요일 == "1_월") %>% 
  #filter(요일 == "6_토") %>% 
  ggplot(aes(amount, fill=성별)) +
  geom_histogram(position="identity", alpha=0.5)


cust_checkout %>% 
  filter(category == "커피/음료전문점") %>% 
  #filter(요일 == "1_월") %>% 
  filter(요일 == "6_토") %>% 
  ggplot(aes(amount, fill=성별)) +
  geom_histogram(position="identity", alpha=0.5)



  ## 위 예제의 출력 결과가 없고 영문 환경인 경우 아래의 명령어 실행
  cust_checkout %>% 
    filter(category == "커피/음료전문점") %>% 
    filter(요일 == "1_Mon") %>% 
    #filter(요일 == "6_Sat") %>% 
    ggplot(aes(amount, fill=성별)) +
    geom_histogram(position="identity", alpha=0.5)
  
  
  cust_checkout %>% 
    filter(category == "커피/음료전문점") %>% 
    #filter(요일 == "1_Mon") %>% 
    filter(요일 == "6_Sat") %>% 
    ggplot(aes(amount, fill=성별)) +
    geom_histogram(position="identity", alpha=0.5)




# [코드 5-135]
cust_checkout %>% 
  filter(category == "커피/음료전문점") %>% 
  ggplot(aes(amount, fill = 성별)) +
  geom_histogram(position = "identity", alpha = 0.5) +
  facet_wrap(vars(요일))



# [코드 5-136]
cust_checkout %>% 
  filter(category == "대형마트") %>% 
  ggplot(aes(y = amount, x = 요일)) +
  geom_boxplot() +
  facet_wrap(vars(연령대_10))


cust_checkout %>% 
  filter(category == "대형마트") %>% 
  ggplot(aes(y = amount, x = 연령대_10)) +
  geom_boxplot() +
  facet_wrap(vars(요일))



# [코드 5-137]
cust_checkout %>% 
  group_by(연령대_10, 성별, category) %>% 
  summarise(total = sum(amount)) %>% 
  mutate(p = total / sum(total)) %>% 
  ggplot(aes(category, p)) +
  geom_col() +
  facet_grid(rows = vars(연령대_10), cols = vars(성별) ) +
  theme(axis.text.x = element_text(angle = 90, vjust = 0.2))



# [코드 5-138]
cust_checkout %>% 
  group_by(요일, 결제시간) %>% 
  summarise(n = n()) %>%
  ggplot(aes(요일, 결제시간, fill = n)) +
  geom_tile() +
  scale_fill_distiller(palette = "YlGnBu", direction = 1)



# [코드 5-139]
cust_checkout %>% 
  group_by(요일, 결제시간, 연령대_10) %>% 
  summarise(n = n()) %>%
  ggplot(aes(요일, 결제시간, fill = n)) +
  geom_tile() +
  scale_fill_distiller(palette = "YlGnBu", direction = 1) +
  facet_wrap(vars(연령대_10))





# 5.16 밥 먹고 두 시간 안에 어떤 업종에서 결제를 많이 할까요? -----------------------------------


# [코드 5-140]
checkout_cust_month = checkout %>% 
  mutate(datetime = strptime(datetime, "%Y-%m-%d %H:%M:%S"),
         month = format(datetime, format="%m")) %>% 
  group_by(cust_id, month) %>% 
  summarise(total = sum(amount)) 
checkout_cust_month



# [코드 5-141]
checkout_cust_month %>% 
  mutate(total_m1 = lag(total)) %>% 
  filter(cust_id %in% c("C10040","C10041","C10042"))



# [코드 5-142]
cust_checkout_grp = checkout_cust_month %>% 
  mutate(total_m1 = lag(total)) %>% 
  filter(month == "06") %>% 
  mutate(grp = if_else(total > total_m1 | is.na(total_m1), "실적증가", "실적감소"))
cust_checkout_grp



# [코드 5-143]
merge(customer %>% select(cust_id), cust_checkout_grp, by = "cust_id", all.x = T) %>% 
  mutate(grp = if_else(is.na(grp), "무실적", grp)) %>% 
  group_by(grp) %>% 
  summarise(n = n())



# [코드 5-144]
checkout_cust_month %>% 
  ungroup() %>% 
  slice_head(n = 10) %>% 
  slice_sample(n = 10)


checkout_cust_month %>% 
  ungroup() %>% 
  slice_head(n = 10) %>% 
  slice_sample(n = 10) %>% 
  mutate(total_m1 = lag(total))



# [코드 5-145]
checkout_prev = checkout %>% 
  mutate( datetime = strptime(datetime, "%Y-%m-%d %H:%M:%S")) %>% 
  arrange(cust_id, datetime) %>% 
  group_by(cust_id) %>% 
  mutate( cate_prev = lag(category),
          dt_diff = datetime - lag(datetime)) 

checkout_prev %>% 
  select(datetime, cust_id, category, cate_prev, dt_diff)



# [코드 5-146]
checkout %>%
  mutate(datetime = strptime(datetime, "%Y-%m-%d %H:%M:%S")) %>%
  arrange(cust_id, datetime) %>%
  ungroup() %>% 
  mutate(cate_prev = lag(category),
         dt_diff = datetime - lag(datetime)) %>% 
  select(datetime, cust_id, category, cate_prev, dt_diff) %>% 
  slice(27:30)



# [코드 5-147]
checkout_prev %>%
  filter(dt_diff <= 7200, !is.na(cate_prev)) %>% 
  select(category, cate_prev)



# [코드 5-148]
checkout_prev %>%
  filter(dt_diff <= 7200, !is.na(cate_prev)) %>% 
  group_by(cate_prev, category) %>% 
  summarise(n = n())



# [코드 5-149]
agg = checkout_prev %>%
  filter(dt_diff <= 7200, !is.na(cate_prev)) %>% 
  group_by(cate_prev, category) %>% 
  summarise(n = n()) %>% 
  mutate(p = n / sum(n))


agg %>% 
  filter(cate_prev == "기타음식점")



# [코드 5-150]
agg %>% 
  ggplot(aes(cate_prev, n, fill = category))+
  geom_col(position = "fill") +
  theme(axis.text.x = element_text(angle = 90, vjust = 0.1, hjust = 1))


agg %>% 
  ggplot(aes(cate_prev, category, fill = p))+
  geom_tile() +
  scale_fill_distiller(palette = "YlGnBu", direction = 1) +
  theme(axis.text.x = element_text(angle = 90, vjust = 0.1, hjust = 1))






# End of script