  # __________________________________________________________ #
  #                                |                           #
  #  이토록 쉬운                   |     "데이터는 답이다"     #
  #  데이터 요약과 시각화 with R   |       _                   #
  # ______________________________ |    __| |   __ _   _ __    #
  #                                |   / _` |  / _` | | '_ \   #
  #  임경덕                        |  | (_| | | (_| | | |_) |  # 
  #   - dataartproject@gmail.com   |   \__,_|  \__,_| | .__/   #
  #   - http://dataartproject.com  |                  |_|      #
  # _______________________________|__________________________ #



# Appendix | 부록




# A.2 RStudio의 프로젝트 기능 활용 -------------------------------------------------


# [코드 A-3]
getwd()




# [코드 A-4]
  ## 아래의 코드는 예시로 실행할 필요 없습니다.
  ## 코드에서 활용한 sample.csv 파일은 data/practice 폴더에 있습니다.
  ## 각자 분석 환경에서 프로젝트 폴더의 위치에 따라 경로를 직접 지정하면 됩니다.

# read.csv("C:/Users/dap/Desktop/MyProject/sample.csv")




# [코드 A-5]
  ## 예제 데이터를 활용할 수도 있지만 관심 데이터를 직접 활용하는 것을 추천합니다.

# read.csv("sample.csv")




# A.3 tidyr 패키지를 활용한 전처리 --------------------------------------------------


# [코드 A-6]
# install.packages("tidyr")
library(tidyr)
library(dplyr)



# [코드 A-7]
checkout = read.csv("data/checkout.csv", fileEncoding = "UTF-8")
checkout %>% 
  tibble()



# [코드 A-8]
checkout %>% 
  group_by(category) %>% 
  filter(category %in% c("대형마트", "백화점")) %>% 
  summarise(n_cust = n_distinct(cust_id),
            total  = sum(amount) / n_cust) 



# [코드 A-9]
checkout %>% 
  group_by(cust_id, category) %>% 
  summarise(total = sum(amount)) 



# [코드 A-10]
checkout %>% 
  summarise(n_cust = n_distinct(cust_id),
            n_cate = n_distinct(category))

8713*24



# [코드 A-11]
checkout %>% 
  group_by(cust_id, category) %>% 
  summarise(total = sum(amount)) %>% 
  # ungroup() %>% 
  complete(cust_id, category)



# [코드 A-12]
checkout %>% 
  group_by(cust_id, category) %>% 
  summarise(total = sum(amount)) %>% 
  ungroup() %>% 
  complete(cust_id, category, fill = list(total = 0)) 



# [코드 A-13]
data_na = read.csv("data/practice/na_data.csv")           
data_na



# [코드 A-14]
data_na %>% 
  mutate(referral = if_else(is.na(referral), "unknown", referral),
         amount   = if_else(is.na(amount),  0L, amount))



# [코드 A-15]
data_na %>% 
  replace_na(list(referral = "unknown",
                  amount   = 0))



# [코드 A-16]
data_na2 = read.csv("data/practice/na_data2.csv")           
data_na2



# [코드 A-17]
data_na2 %>%
  fill(amount)



# [코드 A-18]
data_na2 %>%
  group_by(id) %>% 
  fill(amount, .direction = "up")


data_na2 %>%
  group_by(id) %>% 
  fill(amount, .direction = "updown")



# [코드 A-19]
result = checkout %>% 
  mutate(dow = format(strptime(datetime, format="%F %T"), format="%u_%a")) %>% 
  group_by(dow, category) %>% 
  summarise(total = sum(amount))

result



# [코드 A-20]
result_wide = result %>% 
  spread(category, total)

result_wide %>% 
  tibble()



# [코드 A-21]
result_wide = result %>% 
  spread(category, total, fill = 0)

result_wide %>% 
  tibble()



# [코드 A-22]
result_wide %>% 
  gather("업종", "이용금액", 2:25)







# A.4 파이프 연산자의 추가적인 활용 ----------------------------------------------------


# [코드 A-23]
checkout = read.csv("data/checkout.csv", fileEncoding = "UTF-8")

checkout %>% 
  group_by(category) %>% 
  summarise(n = n(), total = sum(amount)) %>% 
  slice_max(total, n=3)



# [코드 A-24]
ifelse(10>1, "참", "거짓")



# [코드 A-25]
(10>1) %>% 
  ifelse("참", "거짓")



# [코드 A-26]
(10>1) %>% 
  ifelse(., "참", "거짓")



# [코드 A-27]
"새로운 참값" %>% 
  ifelse((10>1), ., "거짓")



# [코드 A-28]
checkout %>% 
  group_by(category) %>% 
  summarise(n_whole     = nrow(.), 
            n           = n(), 
            total_whole = sum(.$amount), 
            total       = sum(amount)) %>% 
  slice_max(total, n = 3)



# [코드 A-29]
checkout %>% 
  group_by(category) %>% 
  summarise(prop_n=n()/nrow(.), prop_total=sum(amount)/sum(.$amount)) %>% 
  slice_max(prop_total, n=3)



# [코드 A-30]
customer = read.csv("data/customer.csv", fileEncoding="UTF-8")


checkout %>% 
  group_by(cust_id) %>% 
  summarise(total = sum(amount)) %>% 
  merge(., customer, by = "cust_id", all.x = T) %>% 
  summarise(n = n())


checkout %>% 
  group_by(cust_id) %>% 
  summarise(total = sum(amount)) %>% 
  merge(customer, ., by = "cust_id", all.x = T) %>% 
  summarise(n = n())






# A.5 ggplot2 패키지의 주요 그래프 속성 변경 함수 ----------------------------------------


# [코드 A-31]
checkout = read.csv("data/checkout.csv", fileEncoding = "UTF-8")
hist(checkout$amount)



# [코드 A-32]
library(ggplot2)
library(dplyr)
checkout %>% 
  ggplot(aes(x = amount)) + geom_histogram()



# [코드 A-33]
qplot(x = checkout$amount, geom = "histogram")
qplot(x = amount, data = checkout, geom = "histogram")



# [코드 A-34]
checkout %>% 
  mutate(date = as.Date(datetime, format="%F %T")) %>% 
  group_by(date) %>% 
  summarise(total = sum(amount)) %>% 
  qplot(x = date, y = total, data = ., geom = "line")



# [코드 A-35]
checkout %>% 
  filter(category %in% c("백화점", "대형마트", "슈퍼마켓", "전자상거래")) %>% 
  group_by(category) %>% 
  summarise(total = sum(amount)) %>% 
  ggplot(aes(category, total)) + 
  geom_col() 



# [코드 A-36]
p = checkout %>% 
  filter(category %in% c("백화점","대형마트","슈퍼마켓","전자상거래")) %>% 
  group_by(category) %>% 
  summarise(total = sum(amount)) %>% 
  ggplot(aes(reorder(category, -total), total)) + 
  geom_col()
p



# [코드 A-37]
p +coord_flip()



# [코드 A-38]
p + scale_y_continuous(labels = scales::comma)



# [코드 A-39]
p + scale_y_continuous(limits = c(0, 1000000000),
                       n.breaks = 10,
                       labels = scales::comma)



# [코드 A-40]
p + scale_y_continuous(limits = c(0, 1000000000),
                       breaks = c(0, 10^8, 2*10^8, 5*10^8, 10^9),
                       labels = c("0억", "1억", "2억", "5억", "10억"))



# [코드 A-41]
p + ylim(0, 10^9)



# [코드 A-42]
p2 = checkout %>% 
  mutate(date = as.Date(datetime, format="%F %T")) %>% 
  group_by(date) %>% 
  summarise(total = sum(amount)) %>% 
  qplot(x = date, y = total, data = ., geom = "line")  

p2 + scale_x_date(date_breaks = "3 days", 
                  date_labels = "%m/%d")



# [코드 A-43]
agg = checkout %>% 
  mutate(dow = format(strptime(datetime, format = "%F %T"), "%u")) %>% 
  group_by(dow, category) %>% 
  summarise(n = n()) 

p3 = agg %>% 
  ggplot(aes(dow, category, fill = n)) + geom_tile()
p3



# [코드 A-44]
p3 + scale_fill_gradient(low="white", high=rgb(240,40,0, maxColorValue=255))



# [코드 A-45]
RColorBrewer::display.brewer.all()



# [코드 A-46]
p3 + scale_fill_distiller(palette = "Blues", direction = 1)



# [코드 A-47]
agg %>% 
  filter(category %in% c("대형마트", "백화점", "병의원")) %>% 
  ggplot(aes(dow, n, fill = category)) + 
  geom_col(position = "dodge") +
  scale_fill_brewer(palette = "Set1")



# [코드 A-48]  
p4 = p3 + labs(title = "요일/업종별 결제건수 요약",
               subtitle = "전체 결제 건수 :  199,664(건)",
               caption = "기간 : 05/25 ~ 06/07",
               fill = "건수")

p4



# [코드 A-49]
p3 + ggtitle("시각화 결과")



# [코드 A-50]
p5 = p4 + xlab("요일") + ylab("업종") 

p5



# [코드 A-51]
p5 + theme_minimal()

  # macOS에서는 한글폰트 지정 명령어 추가(아래도 동일)
  p5 + theme_minimal() + theme(text = element_text(family='AppleGothic'))



# [코드 A-52]
p5 + theme(title = element_text(size = 20),
           text  = element_text(size = 12),
           line  = element_line(size = 2))




# [코드 A-53]
p5 + theme(title         = element_text(size = 20, face = "bold", ),
           plot.subtitle = element_text(size = 12, face = "italic"),
           axis.title    = element_text(size = 14),
           legend.title  = element_blank(),
           axis.text.y   = element_text(size = 12, angle = 30))



# [코드 A-54]
install.packages(c("jsonlite", "curl"))
install.packages("showtext")

library(showtext)
font_add_google("Nanum Gothic", "nanumgothic")



# [코드 A-55]
font_add("myfont", "data/practice/batang.ttc")
sysfonts::font_families()



# [코드 A-56]
showtext_auto()

p6 = p5 + theme(title = element_text(size = 20, face = "bold", family = "nanumgothic"),
                plot.subtitle = element_text(size = 12, face = "italic", family = "myfont"))
p6

ggsave("output_plot.png", p6)








# A.6 간단한 정규 표현식 ----------------------------------------------------------

# [코드 A-57]
checkout = read.csv("data/checkout.csv", fileEncoding = "UTF-8")

checkout %>% filter(category == "백화점")
checkout %>% filter(category %in% c("백화점", "대형마트"))
checkout %>% filter(category %in% c("한식", "일식"))



# [코드 A-58]
text = c("통계", "데이터", "데이터분석", "R과 통계")
text


# [코드 A-59]
text %in% c("통계", "데이터")



# [코드 A-60]
grepl("통계", text)



# [코드 A-61]
grepl("통계|데이터", text)



# [코드 A-62]
grep("통계", text)



# [코드 A-63]
grep("통계", text, value=T)
grep("통계", text, value=T, invert=T)



# [코드 A-64]
checkout %>% 
  filter( grepl("한식|일식", category)) %>% 
  count(category)



# [코드 A-65]
text
grepl("[가-싷]{2}", text)



# [코드 A-66]
grepl("[[:blank:]]", text)
grepl("[^가-힣 ]", text)



# [코드 A-67]
text2 = c("가나다123ABC", "XYZ.9", "54321", "A/Z", "10 가")
text2



# [코드 A-68]
grep("[가-힣]", text2, value = T)
grep("[^가-힣]", text2, value = T)
grep("[가-힣]", text2, value = T, invert = T)



# [코드 A-69]
grepl("[가-힣]", text2)
!grepl("[가-힣]", text2)



# [코드 A-70]
grep("^[가-힣]", text2, value = T)
grep("[가-힣]$", text2, value = T)



# [코드 A-71]
checkout %>% 
  filter(grepl("[[:punct:]]", category)) %>% 
  count(category)



# [코드 A-72]
customer = read.csv("data/customer.csv", fileEncoding = "UTF-8")

customer %>% 
  filter(grepl("^[4-9]", age)) %>% 
  summarise(min(age))



# [코드 A-73]
customer %>% 
  filter(grepl("[가-힣]{5,}", address)) %>% 
  select(1:4) %>% 
  head()



# [코드 A-74]
text2
gsub("가나다|ABC", "하하하", text2)



# [코드 A-75]
grep("[0-9]{3,}", text2)
gsub("[0-9]{3,}", "---", text2)



# [코드 A-76]
gsub("[[:punct:]]", "", text2)



# [코드 A-77]
gsub("[^[:alnum:]]", "", text2)



# [코드 A-78]
longtext = "어쩌고저쩌고 .@# product_id=P02930&ABC 123"
longtext



# [코드 A-79]
grepl(".*product_id=P[0-9]{5}&.*", longtext)



# [코드 A-80]
gsub(".*product_id=(P[0-9]{5})&.*", "\\1", longtext)



# [코드 A-81]
customer %>% 
  slice_head(n = 3) %>% 
  select(1:4)



# [코드 A-82]
customer %>% 
  select(1:4) %>% 
  mutate(roadname = gsub("[가-힣]{2,} ([가-힣0-9]{2,}) [0-9]{1,}", "\\1", address)) %>% 
  slice_head(n = 3)



# [코드 A-83]
merchant = read.csv("data/merchant.csv", fileEncoding="UTF-8")
merchant %>% 
  filter(category == "편의점") %>% 
  select(1:3) %>% 
  slice_head(n = 3)



# [코드 A-84]
merchant %>% 
  filter(category=="편의점") %>% 
  mutate(brand = gsub("^([[:alpha:]]{2,}) .*", "\\1", name)) %>% 
  count(brand)






# A.7 변수 형식 ---------------------------------------------------------------


# [코드 A-85]
sample = read.csv("data/sample2.csv")
sample

sample %>% 
  str(sample)



# [코드 A-86]
sample %>% 
  tibble()



# [코드 A-87]
read.csv("data/sample2.csv", stringsAsFactors = T) %>% 
  str()



# [코드 A-88]
sample = sample %>% 
  mutate(ID = as.character(ID))

sample %>% 
  tibble()



# [코드 A-89]
sample = sample %>% 
  mutate(GENDER = as.factor(GENDER))

sample %>% 
  tibble()



# [코드 A-90]
sample %>% 
  mutate(GENDER = ifelse(GENDER=="M", "남", "여"))



# [코드 A-91]
levels(sample$GENDER)



# [코드 A-92]
levels(sample$GENDER) = c("여", "남")
sample





# End of script